ig.module(
	'game.main'
).requires(
	'impact.game',
	'impact.font',
	'plugins.timeslower',
	'plugins.screenfader',
	'plugins.screenshaker',
	'plugins.scene_manager',
	'plugins.touchcontrols',
	'plugins.utilities',
	'game.entities.block',
	'game.entities.titlegraphic',
	'game.entities.floatingtext',
	'game.entities.particles',
	'game.scenes.menuscene',
	'game.scenes.playingscene',
	'game.scenes.gameoverscene',
	'game.scenes.winscene',
	'game.scenes.adscene',
	'game.misc.background',
	'game.misc.level',
	'game.misc.playgrid',
	'game.misc.blockgenerator'
).defines(function() {
	MyGame = ig.Game.extend({
		scoreFont48: new ig.Font('media/boogaloo42.png'),
		backgroundMusic: new ig.Sound('media/audio/music.*'),
		sceneManager: new SceneManager(),
		touchController: null,
		screenFader: new ScreenFader(),
		background: null,
		playGrid: null,
		blockGenerator: null,
		score: 0,
		currentLevel: 1,
		gravity: 100,
		paused: false,
		unpauseButton: null,
		levels: [
			new Level(0, 1, 1),
			new Level(25, 0.5, 2),
			new Level(50, 0.5, 3),
			new Level(75, 0.5, 3),
			new Level(125, 0.5, 4),
			new Level(200, 0.5, 4),
			new Level(325, 0.4, 5),
			new Level(525, 0.4, 5),
			new Level(850, 0.4, 6),
			new Level(1375, 0.3, 6),
			new Level(2225, 0.3, 7),
			new Level(3600, 0.2, 7),
		],
		scoreString: {
			en: 'SCORE:'
		},
		init: function() {
			if (ig.ua.mobile) {
				window.scrollTo(0, 0);
			}
			ig.input.bind(ig.KEY.MOUSE1, 'click');

			this.touchController = new TouchControls(ig.system.canvas);
			this.sceneManager.pushScene(new MenuScene());
			this.background = new Background();
			this.playGrid = new PlayGrid(8, 46);
			this.blockGenerator = new BlockGenerator(8, 392);

			ig.music.volume = 0.2;
			ig.music.add(this.backgroundMusic);
			ig.music.play();
		},
		update: function() {
			if (ig.ua.mobile) {
				window.scrollTo(0, 0);
			}
			this.parent();
			this.sceneManager.updateScene();
			this.background.update();
			this.playGrid.update();
			this.blockGenerator.update();
			if (this.currentLevel >= this.levels.length) {
				var endLevel = this.levels[this.levels.length - 1];
				this.levels.push(new Level(endLevel.next + 800, endLevel.speed, endLevel.height));
			}
			if (ig.input.pressed('pause')) {
				this.pause();
			}
		},
		draw: function() {
			if (this.clearColor) {
				ig.system.clear(this.clearColor);
			}
			this._rscreen.x = ig.system.getDrawPos(this.screen.x) / ig.system.scale;
			this._rscreen.y = ig.system.getDrawPos(this.screen.y) / ig.system.scale;
			var mapIndex;
			for (mapIndex = 0; mapIndex < this.backgroundMaps.length; mapIndex++) {
				var map = this.backgroundMaps[mapIndex];
				if (map.foreground) {
					break;
				}
				map.setScreenPos(this.screen.x, this.screen.y);
				map.draw();
			}
			this.background.draw();
			this.drawEntities();
			for (mapIndex; mapIndex < this.backgroundMaps.length; mapIndex++) {
				var map = this.backgroundMaps[mapIndex];
				map.setScreenPos(this.screen.x, this.screen.y);
				map.draw();
			}
			this.drawScore();
			this.sceneManager.drawScene();
			this.playGrid.draw();
			this.blockGenerator.draw();
			this.touchController.draw();
			this.screenFader.draw();
		},
		drawBackgroundLines: function() {},
		drawScore: function() {
			this.scoreFont48.draw(this.scoreString.en + this.score, 0, -7, ig.Font.ALIGN.LEFT);
		},
		makeNewLevel: function() {
			var speed = this.levels[this.levels.length - 1].speed;
		},
		restartGame: function() {
			this.score = 0;
			this.currentLevel = 1;
			this.playGrid.clearGrid();
			this.playGrid.killAllBlocks();
			this.blockGenerator.clearArray();
			this.sceneManager.popToRootScene();
			this.sceneManager.pushScene(new PlayingScene());
		},
		pause: function() {
			if (this.paused) {
				this.unpause();
			} else {
				this.paused = true;
				if (this.blockGenerator) {
					this.blockGenerator.stopBlockTimer();
				}
				if (this.playGrid) {
					this.playGrid.paused = true;
				}
				this.unpauseButton = this.touchController.addButton(ButtonUnpause, 60, 200);
			}
		},
		unpause: function() {
			this.paused = false;
			if (this.blockGenerator) {
				this.blockGenerator.startBlockTimer();
			}
			if (this.playGrid) {
				this.playGrid.paused = false;
			}
			this.touchController.removeButton(this.unpauseButton);
		}
	});
	TouchControls.inject({
		touchFlare: function(x, y) {
			for (var i = 0; i < 4; i++) {
				ig.game.spawnEntity(EntityParticle, x - 16 + randomRange(-5, 5), y - 16 + randomRange(-5, 5), {
					vel: {
						x: randomRange(-40, 40),
						y: randomRange(-40, 40)
					}
				});
			}
		}
	});
	ig.language = 'en', ig.main('#canvas', MyGame, 60, 320, 480, 1);
});