ig.module('game.misc.blockgenerator').defines(function() {
	BlockGenerator = ig.Class.extend({
		_debug: false,
		pos: {
			x: 0,
			y: 0
		},
		cellSize: 38,
		array_size: 8,
		blocksArray: [],
		blockSpawnSpeed: 0.5,
		blockSpawnTimer: null,
		randomBlocks: [],
		init: function(x, y) {
			this.pos.x = (x) ? x : 0;
			this.pos.y = (y) ? y : 0;
			this.randomBlocks.push(EntityOakblock);
			this.randomBlocks.push(EntityFireblock);
			this.randomBlocks.push(EntityLightningblock);
		},
		update: function() {
			if (this.blockSpawnTimer && this.blockSpawnTimer.delta() >= 0) {
				if (this.blocksArray.length < this.array_size) {
					var newBlock = ig.game.spawnTouchEntity(this.randomBlocks.random(), 0, 0);
					var newIndex = this.blocksArray.push(newBlock) - 1;
					newBlock.i = newIndex;
					newBlock.pos.x = this.getCellX(newBlock.i);
					newBlock.pos.y = this.getCellY(newBlock.i);
					this.blockSpawnTimer.reset();
				} else {
					if (!this.checkIfGameOver()) {
						ig.game.playGrid.shiftAllBlocksUpOne();
						this.dumpToGrid();
						this.blockSpawnTimer.reset();
					} else {
						ig.game.sceneManager.pushScene(new GameOverScene());
					}
				}
			}
		},
		draw: function() {
			if (this._debug) {
				var ctx = ig.system.context;
				ctx.strokeStyle = 'rgb(255,255,0)';
				ctx.strokeRect(this.pos.x, this.pos.y, this.cellSize * this.array_size, this.cellSize);
			}
		},
		clearArray: function() {
			var block = this.blocksArray.pop();
			while (block) {
				block = this.blocksArray.pop();
			}
		},
		startBlockTimer: function(time) {
			var spawnSpeed = (time) ? time : this.blockSpawnSpeed;
			this.blockSpawnTimer = new ig.Timer(spawnSpeed);
		},
		stopBlockTimer: function() {
			this.blockSpawnTimer = null;
		},
		getCellX: function(i) {
			return this.pos.x + i * this.cellSize;
		},
		getCellY: function(i) {
			return this.pos.y;
		},
		dumpToGrid: function() {
			for (var i = 0; i < this.blocksArray.length; i++) {
				var block = this.blocksArray[i];
				ig.game.playGrid.grid[ig.game.playGrid.grid_height - 1][block.i] = block;
				block.j = ig.game.playGrid.grid_height - 1;
				block.pos.x = ig.game.playGrid.getCellX(block.j, block.i);
				block.pos.y = ig.game.playGrid.getCellY(block.j, block.i);
			}
			this.clearArray();
		},
		checkIfGameOver: function() {
			var isGameOver = ig.game.playGrid.checkIfGameOver();
			if (isGameOver) {
				ig.log('game is over');
				this.stopBlockTimer();
			}
			return isGameOver;
		},
	});
});
