ig.module('game.misc.playgrid').requires('game.entities.block', 'impact.image').defines(function() {
	PlayGrid = ig.Class.extend({
		sound: new ig.Sound('media/audio/coin.*'),
		_debug: false,
		pos: {
			x: 0,
			y: 0
		},
		grid: [],
		grid_width: 8,
		grid_height: 9,
		cellSize: 38,
		cellSpacing: 0,
		matchingBlocksArray: [],
		someBlocksDestroyed: false,
		paused: false,
		outline: new ig.Image('media/playgrid_outline.png'),
		outlineOffset: {
			x: -15,
			y: 37
		},
		showOutline: false,
		init: function(x, y) {
			this.pos.x = (x) ? x : 0;
			this.pos.y = (y) ? y : 0;
			this.initGrid();
			ig.log('size = ' + this.cellSize * this.grid_width + ", " + this.cellSize * this.grid_height);
		},
		update: function() {
			if (this.someBlocksDestroyed) {
				this.shiftBlocksDown();
				this.shiftBlocksSideways();
				this.someBlocksDestroyed = false;
			}
		},
		draw: function() {
			if (this._debug) {
				var ctx = ig.system.context;
				ctx.strokeStyle = 'rgb(255,0,0)';
				ctx.strokeRect(this.pos.x, this.pos.y, this.cellSize * this.grid_width, this.cellSize * this.grid_height);
			}
			if (this.showOutline) {
				this.outline.draw(this.outlineOffset.x, this.outlineOffset.y);
			}
		},
		placeBlocksRandom: function(height) {
			var blockRandomizer = [EntityOakblock, EntityFireblock, EntityLightningblock]
			var height = (height) ? height : 1;
			if (height < this.grid_height + 1) {
				for (var j = this.grid_height - 1; j > this.grid_height - 1 - height; j--) {
					for (var i = 0; i < this.grid_width; i++) {
						var x = this.getCellX(j, i);
						var y = this.getCellY(j, i);
						this.grid[j][i] = ig.game.spawnTouchEntity(blockRandomizer.random(), x, y, {
							j: j,
							i: i
						});
					}
				}
			} else {
				ig.log('error, height is higher than grid height');
			}
		},
		getCellX: function(j, i) {
			return this.pos.x + i * this.cellSize + i * this.cellSpacing;
		},
		getCellY: function(j, i) {
			return this.pos.y + j * this.cellSize + j * this.cellSpacing;
		},
		initGrid: function() {
			while (this.grid.push([]) < this.grid_height);
			for (var j = 0; j < this.grid_height; j++) {
				while (this.grid[j].push(null) < this.grid_width);
			}
		},
		clearGrid: function() {
			for (var j = 0; j < this.grid.length; j++) {
				for (var i = 0; i < this.grid[j].length; i++) {
					if (this.grid[j][i]) {
						this.grid[j][i].kill();
					}
					this.grid[j][i] = null;
				}
			}
		},
		clearGridGameOver: function() {
			var skipColumn = null;
			for (var n = 0; n < this.grid_width; n++) {
				if (this.grid[0][n]) {
					skipColumn = n;
				}
			}
			for (var j = 0; j < this.grid.length; j++) {
				for (var i = 0; i < this.grid[j].length; i++) {
					if (i != skipColumn) {
						if (this.grid[j][i]) {
							this.grid[j][i].kill();
						}
						this.grid[j][i] = null;
					}
				}
			}
		},
		makeBlocksInert: function() {
			for (var j = 0; j < this.grid.length; j++) {
				for (var i = 0; i < this.grid[j].length; i++) {
					this.grid[j][i] = null;
				}
			}
		},
		killAllBlocks: function() {
			for (var i = 0; i < ig.game.entities.length; i++) {
				ig.game.entities[i].kill();
			}
		},
		checkAndExplodeBlocks: function(block) {
			if (!this.paused) {
				var block = block;
				this.checkIfSurroundingBlocksMatch(block);
				if (this.matchingBlocksArray.length >= 2) {
					this.explodeMatchingBlocks();
				}
				while (this.matchingBlocksArray.length) {
					this.matchingBlocksArray.pop();
				}
				this.removeCheckedFlag();
				this.someBlocksDestroyed = true;
			}
		},
		checkIfSurroundingBlocksMatch: function(block) {
			var block = block;
			this.matchingBlocksArray.push(block);
			block.hasBeenChecked = true;
			var jprime = block.j;
			var iprime = block.i - 1;
			if (this.checkBlockForSameColor(block, jprime, iprime)) {
				this.checkIfSurroundingBlocksMatch(this.grid[jprime][iprime]);
			}
			jprime = block.j - 1;
			iprime = block.i;
			if (this.checkBlockForSameColor(block, jprime, iprime)) {
				this.checkIfSurroundingBlocksMatch(this.grid[jprime][iprime]);
			}
			jprime = block.j;
			iprime = block.i + 1;
			if (this.checkBlockForSameColor(block, jprime, iprime)) {
				this.checkIfSurroundingBlocksMatch(this.grid[jprime][iprime]);
			}
			jprime = block.j + 1;
			iprime = block.i;
			if (this.checkBlockForSameColor(block, jprime, iprime)) {
				this.checkIfSurroundingBlocksMatch(this.grid[jprime][iprime]);
			}
		},
		checkBlockForSameColor: function(currentBlock, jprime, iprime) {
			var isSameColor = false;
			if (this.checkIfInBounds(jprime, iprime)) {
				if (this.grid[jprime][iprime]) {
					var nextBlock = this.grid[jprime][iprime];
					if (nextBlock.color == currentBlock.color && !nextBlock.hasBeenChecked) {
						isSameColor = true;
					}
				}
			}
			return isSameColor;
		},
		checkIfInBounds: function(j, i) {
			var returnValue = false;
			if (j >= 0 && j < this.grid_height && i >= 0 && i < this.grid_width) {
				returnValue = true;
			}
			return returnValue;
		},
		explodeMatchingBlocks: function() {
			var numberOfBlocks = 0;
			var x = 0;
			var y = 0;
			if (this.matchingBlocksArray.length > 1) {
				x = this.matchingBlocksArray[0].pos.x;
				y = this.matchingBlocksArray[0].pos.y;
				numberOfBlocks = this.matchingBlocksArray.length;
				while (this.matchingBlocksArray.length > 0) {
					this.sound.play();
					var block = this.matchingBlocksArray.pop();
					this.grid[block.j][block.i] = null;
					block.kill();
				}
			} else {
				while (this.matchingBlocksArray.length > 0) {
					this.matchingBlocksArray.pop();
				}
			}
			if (numberOfBlocks > 2) {
				ig.game.background.pulseRays();
				var score = -5 + 2 * numberOfBlocks;
				ig.game.spawnEntity(EntityFloatingtext, x + 20, y - 50, {
					string: "+" + score
				});
				ig.game.score += score;
				if (numberOfBlocks > 4) {
					var x = randomRange(-75, 75);
					var y = randomRange(-75, 75);
				}
			}
		},
		shiftBlocksDown: function() {
			for (var i = 0; i < this.grid_width; i++) {
				var emptyJValues = [];
				for (var j = this.grid_height - 1; j >= 0; j--) {
					if (this.grid[j][i]) {
						if (emptyJValues.length > 0) {
							this.moveBlockTo(this.grid[j][i], emptyJValues.shift(), i);
							emptyJValues.push(j);
						}
					} else {
						emptyJValues.push(j);
					}
				}
			}
		},
		shiftBlocksSideways: function() {},
		removeCheckedFlag: function() {
			for (var n = 0; n < ig.game.entities.length; n++) {
				if (ig.game.entities[n].hasBeenChecked) {
					ig.game.entities[n].hasBeenChecked = false;
				}
			}
		},
		moveBlockTo: function(block, j, i) {
			if (this.grid[j][i]) {
				ig.log('grid location not empty, cannot move block: j=' + j + ', i=' + i);
			} else {
				this.grid[j][i] = block;
				this.grid[block.j][block.i] = null;
				block.j = j;
				block.i = i;
				block.pos.x = this.getCellX(j, i);
				block.pos.y = this.getCellY(j, i);
			}
		},
		shiftAllBlocksUpOne: function() {
			for (var j = 1; j < this.grid_height; j++) {
				for (var i = 0; i < this.grid_width; i++) {
					var block = this.grid[j][i];
					if (block) {
						this.moveBlockTo(block, j - 1, i);
					}
				}
			}
		},
		checkIfGameOver: function() {
			var isGameOver = false;
			for (var i = 0; i < this.grid_width; i++) {
				if (this.grid[0][i]) {
					isGameOver = true;
				}
			}
			return isGameOver;
		}
	});
});