ig.module(
	'plugins.scene_manager'
).requires(
	'impact.game'
).defines(function() {
	Scene = ig.Class.extend({
		title: null,
		sceneManager: null,
		font: null,
		center: {
			x: 0,
			y: 0
		},
		suspendedHere: false,
		init: function(title) {
			this.title = title || null;
			this.center.x = ig.system.width * 0.5;
			this.center.y = ig.system.height * 0.5;
		},
		update: function() {},
		draw: function() {},
		onExit: function() {},
		onEntry: function() {},
		isTopScene: function() {
			if (this.sceneManager) {
				var currentScene = this.sceneManager.sceneStack.last();
				return (this == currentScene);
			}
		},
	});
	SceneManager = ig.Class.extend({
		sceneStack: [],
		staticInstantiate: function() {
			if (SceneManager.instance == null) {
				return null;
			} else {
				return SceneManager.instance;
			}
		},
		init: function() {
			SceneManager.instance = this;
		},
		updateScene: function() {
			var currentScene = this.sceneStack.last();
			if (currentScene instanceof Scene) {
				currentScene.update();
			}
		},
		drawScene: function() {
			var currentScene = this.sceneStack.last();
			if (currentScene instanceof Scene) {
				currentScene.draw();
			}
		},
		pushScene: function(newScene) {
			newScene.sceneManager = this;
			if (this.sceneStack.length > 0) {
				this.getTopScene().onExit();
			}
			this.sceneStack.push(newScene);
			this.getTopScene().onEntry();
			return newScene;
		},
		popScene: function() {
			if (this.sceneStack.length > 1) {
				var oldScene = this.sceneStack.pop(),
					newScene = this.sceneStack.last();
				oldScene.onExit();
				newScene.onEntry();
				return oldScene;
			}
			return null;
		},
		getTopScene: function() {
			return this.sceneStack[this.sceneStack.length - 1];
		},
		popToRootScene: function() {
			this.sceneStack[this.sceneStack.length - 1].onExit();
			this.sceneStack = this.sceneStack.slice(0, 1);
			this.sceneStack[0].onEntry();
		},
		replaceScene: function(newScene) {
			this.sceneStack[this.sceneStack.length - 1].onExit();
			this.sceneStack.splice(-1, 1, newScene);
			newScene.onEntry();
		},
		setScenes: function(newScenes) {
			var numNewScenes = newScenes.length;
			this.getTopScene().onExit();
			this.sceneStack = [];
			for (var i = 0; i < numNewScenes; i++) {
				var newScene = newScenes[i];
				this.pushScene(newScene);
			}
		}
	});
	SceneManager.instance = null;
});