ig.module(
	'plugins.touchcontrols'
).requires(
	'impact.animation',
	'impact.entity',
	'impact.game'
).defines(function() {
	TouchControls = ig.Class.extend({
		_debug: true,
		touchable: false,
		selector: null,
		buttons: [],
		touchEntities: [],
		mobilizer: null,
		staticInstantiate: function() {
			if (TouchControls.instance == null) {
				return null;
			} else {
				return TouchControls.instance;
			}
		},
		joystick: null,
		init: function(selector) {
			var canvas = ig.system.canvas;
			TouchControls.instance = this;
			this.touchable = 'createTouch' in document;
			this.selector = selector;
			this.controlsActivate(selector);

			this.mobilizer = new Mobilizer(canvas.width, canvas.height, 'portrait');
			if (ig.ua.mobile) {
				this.mobilizer.mobilize();
			}
		},
		draw: function() {
			if (this.touchable && this.joystick) {
				this.joystick.draw();
			}
			for (var i = 0; i < this.buttons.length; i++) {
				this.buttons[i].draw();
			}
		},
		update: function() {
			if (this.touchable && this.joystick) {
				this.joystick.update();
			}
			if (ig.ua.mobile) {
				this.mobilizer.update();
			}
		},
		makeJoystick: function(x, y, id) {
			if (!this.joystick) {
				this.joystick = new Joystick(x, y, id);
			}
		},
		addButton: function(type, x, y, settings) {
			var buttonClass = typeof(type) === 'string' ? ig.global[type] : type;
			if (!buttonClass) {
				throw ('cant add button of type ' + type);
			}
			var button = new(buttonClass)(x, y, settings || {});
			this.buttons.push(button);
			return button;
		},
		removeButton: function(button) {
			this.buttons.erase(button);
		},
		clearButtons: function() {
			var goThrough = this.buttons.length;
			for (var i = 0; i < goThrough; i++) {
				if (this.buttons[this.buttons.length - 1 - i] && !this.buttons[this.buttons.length - 1 - i].isPersistent) {
					this.buttons.erase(this.buttons[this.buttons.length - 1 - i]);
					i++;
				}
			}
		},
		controlsActivate: function(selector) {
			if (this.touchable) {
				selector.addEventListener('touchstart', this.onMouseDown, false);
				selector.addEventListener('touchend', this.onMouseUp, false);
				selector.addEventListener('touchmove', this.onTouchMove, false);
			} else {
				ig.log('mouse controls');
				selector.addEventListener('mousedown', this.onMouseDown, false);
				selector.addEventListener('mouseup', this.onMouseUp, false);
			}
		},
		onMouseUp: function(ev) {
			ev.preventDefault();
			if (this.touchable) {
				ig.log('length of ev.changedTouches inside onMouseUp() = ' + ev.changedTouches.length);
			}
			var that = new TouchControls(ig.system.canvas);
			var clickPosition = that.getPosition(ev);
			for (var i = 0; i < that.buttons.length; i++) {
				if (that.touchable) {
					for (var j = 0; j < ev.changedTouches.length; j++) {
						clickPosition = that.getPosition(ev.changedTouches[j]);
						that.buttons[i].checkIfMouseUp(clickPosition.x, clickPosition.y, ev.changedTouches[j].identifier);
					}
				} else {
					that.buttons[i].checkIfMouseUp(clickPosition.x, clickPosition.y);
				}
			}
		},
		onMouseDown: function(ev) {
			ev.preventDefault();
			if (this.touchable) {
				for (var n = 0; n < ev.changedTouches.length; n++) {
					ig.log('onMousedown(), touch id = ' + ev.changedTouches[n].identifier);
				}
			}
			var that = new TouchControls(ig.system.canvas);
			//that.sound.play();
			var clickPosition = that.getPosition(ev);
			ig.log("position mouseDown = " + clickPosition.x + ", " + clickPosition.y);
			for (var i = 0; i < that.buttons.length; i++) {
				if (that.touchable) {
					for (var j = 0; j < ev.changedTouches.length; j++) {
						clickPosition = that.getPosition(ev.changedTouches[j]);
						ig.log("position mouse touch= " + clickPosition.x + ", " + clickPosition.y);
						that.touchFlare(clickPosition.x, clickPosition.y);
						that.buttons[i].checkIfMouseDown(clickPosition.x, clickPosition.y, ev.changedTouches[j].identifier);
					}
				} else {
					that.buttons[i].checkIfMouseDown(clickPosition.x, clickPosition.y);
					that.touchFlare(clickPosition.x, clickPosition.y);
				}
			}
			for (var j = 0; j < that.touchEntities.length; j++) {
				that.touchEntities[j].checkIfClicked(clickPosition.x, clickPosition.y);
			}
		},
		onClick: function(ev) {
			ev.preventDefault();
			var that = new TouchControls(ig.system.canvas);
			var clickPosition = that.getPosition(ev);
			ig.log("position = " + clickPosition.x + ", " + clickPosition.y);
			for (var i = 0; i < that.buttons.length; i++) {
				that.buttons[i].checkIfClicked(clickPosition.x, clickPosition.y);
			}
		},
		onTouchMove: function(ev) {
			ev.preventDefault();
			var that = new TouchControls(ig.system.canvas);
			if (that.joystick) {
				for (var i = 0; i < ev.changedTouches.length; i++) {
					if (ev.changedTouches[i].identifier == that.joystick.touchId) {
						that.joystick.thumbPos = that.getPosition(ev.changedTouches[i]);
						if (that.joystick.thumbPos.x > that.joystick.pos.x + 20) {
							ig.input.actions['right'] = true;
							ig.input.actions['left'] = false;
						} else if (that.joystick.thumbPos.x < that.joystick.pos.x - 20) {
							ig.input.actions['left'] = true;
							ig.input.actions['right'] = false;
						} else {
							ig.input.actions['left'] = false;
							ig.input.actions['right'] = false;
						}
						i = ev.changedTouches.length;
					}
				}
			}
		},
		getPosition: function(ev) {
			var canvas = ig.system.canvas;
			var rect = canvas.getBoundingClientRect();
			var x = ev.clientX - rect.left;
			var y = ev.clientY - rect.top;
			if (ig.ua.mobile) {
				x *= this.mobilizer.getScreenRatioWidth();
				y *= this.mobilizer.getScreenRatioHeight();
			}
			return {
				x: x / ig.system.scale,
				y: y / ig.system.scale
			};
		},
		touchFlare: function(x, y) {}
	});
	TouchEntity = ig.Entity.extend({
		touchable: true,
		checkIfClicked: function(x, y) {
			if (x > this.pos.x && x < this.pos.x + this.size.x && y > this.pos.y && y < this.pos.y + this.size.y) {
				this.pressed();
			}
		},
		pressed: function() {},
		kill: function() {
			this.parent();
			var touchController = new TouchControls(ig.system.canvas);
			touchController.touchEntities.erase(this);
		}
	});
	Mobilizer = ig.Class.extend({
		landscapeOnly: false,
		portraitOnly: false,
		wasPortrait: -1,
		init: function(x, y, orientation) {
			if (orientation && orientation == 'portrait') {
				this.portraitOnly = true;
				this.landscapeOnly = false;
			} else if (orientation && orientation == 'landscape') {
				this.landscapeOnly = true;
				this.portraitOnly = false;
			}
		},
		mobilize: function() {
			var canvas = document.getElementById('canvas');
			if (canvas) {
				canvas.style.width = window.innerWidth + 'px';
				canvas.style.height = window.innerHeight + 'px';
			}
			window.addEventListener('orientationchange', this.checkOrientation, false);
			window.addEventListener('resize', this.checkOrientation, false);
			this.checkOrientation(this);
		},
		getScreenRatioWidth: function() {
			return ig.system.width / window.innerWidth;
		},
		getScreenRatioHeight: function() {
			return ig.system.height / window.innerHeight;
		},
		update: function() {},
		checkOrientation: function(mobilizerRef) {
			var therp = null;
			if (mobilizerRef) {
				therp = mobilizerRef;
			} else {
				therp = ig.game.touchController.mobilizer;
			}
			var isPortrait = (window.innerHeight > window.innerWidth);
			if (isPortrait === therp.wasPortrait) {} else {
				if (!isPortrait) {
					document.getElementById('canvas').style.display = "none";
					document.getElementById('rotate').style.display = "block";
				} else if (isPortrait && !therp.wasPortrait) {
					document.getElementById('canvas').style.display = "block";
					document.getElementById('rotate').style.display = "none";
					var canvas = document.getElementById('canvas');
					if (canvas) {
						canvas.style.width = window.innerWidth + 'px';
						canvas.style.height = window.innerHeight + 'px';
					}
				}
			}
			therp.wasPortrait = isPortrait;
		}
	});
	AButton = ig.Class.extend({
		sound: new ig.Sound('media/audio/button-click.*'),
		name: 'name_empty',
		isPersistent: false,
		action: null,
		pos: {
			x: 0,
			y: 0
		},
		size: {
			x: 1,
			y: 1
		},
		offset: {
			x: 0,
			y: 0
		},
		_debug: false,
		_strokeColor: 'rgba(0,0,255,0.8)',
		_strokeColorPressed: 'rgba(255,0,0,0.8)',
		_strokeSize: 1,
		animSheet: null,
		anims: {},
		currentAnim: null,
		isPressed: false,
		pressedEventId: null,
		quick: true,
		init: function(x, y, settings) {
			this.pos.x = x;
			this.pos.y = y;
			ig.merge(this, settings);
			if (this.animSheet) {
				this.addAnim('regular', 0.1, [0], true);
				this.addAnim('mouseDown', 0.1, [1], true);
			}
		},
		draw: function() {
			if (this._debug) {
				var canvas = ig.system.canvas;
				var context = ig.system.context;
				context.save();
				var color = (!this.isPressed) ? this._strokeColor : this._strokeColorPressed;
				context.strokeStyle = color;
				context.lineWidth = this._strokeSize;
				context.strokeRect(this.pos.x, this.pos.y, this.size.x, this.size.y);
				context.restore();
			}
			if (this.currentAnim) {
				this.currentAnim.draw(this.pos.x - this.offset.x, this.pos.y - this.offset.y);
			}
		},
		addAnim: function(name, frameTime, sequence, stop) {
			if (!this.animSheet) {
				throw ('No animSheet to add the animation ' + name + ' to.');
			}
			var a = new ig.Animation(this.animSheet, frameTime, sequence, stop);
			this.anims[name] = a;
			if (!this.currentAnim) {
				this.currentAnim = a;
			}
			return a;
		},
		pressed: function() {
			ig.input.presses[this.action] = true;
		},
		mouseDown: function() {
			this.isPressed = true;
			if (this.sound) this.sound.play();
			this.currentAnim = this.anims.mouseDown;
			if (this.quick) {
				this.pressed();
			}
		},
		mouseUp: function() {
			if (!this.quick) {
				this.pressed();
			}
			this.isPressed = false;
			this.clearMouseDown();
		},
		clearMouseDown: function() {
			this.isPressed = false;
			this.currentAnim = this.anims.regular;
		},
		checkIfClicked: function(x, y) {
			if (x > this.pos.x && x < this.pos.x + this.size.x && y > this.pos.y && y < this.pos.y + this.size.y) {
				this.pressed();
			}
		},
		checkIfMouseDown: function(x, y) {
			if (x > this.pos.x && x < this.pos.x + this.size.x && y > this.pos.y && y < this.pos.y + this.size.y) {
				this.mouseDown();
			}
		},
		checkIfMouseUp: function(x, y) {
			if (x > this.pos.x && x < this.pos.x + this.size.x && y > this.pos.y && y < this.pos.y + this.size.y && this.isPressed) {
				this.mouseUp();
			}
			this.clearMouseDown();
		}
	});
	Joystick = ig.Class.extend({
		pos: {
			x: 0,
			y: 0
		},
		thumbPos: {
			x: 0,
			y: 0
		},
		color: 'rgba(255,0,0,0.5)',
		thumbColor: 'rgba(0,0,230,0.5)',
		radius: 40,
		thumbRadius: 30,
		strokeSize: 9,
		arc360: Math.PI * 2,
		touchId: null,
		init: function(x, y, id) {
			this.pos.x = x;
			this.pos.y = y;
			this.thumbPos = this.pos;
			this.touchId = id;
		},
		draw: function() {
			this.drawBase();
			this.drawThumb();
		},
		update: function() {},
		drawBase: function() {
			var canvas = ig.system.canvas;
			var context = ig.system.context;
			context.beginPath();
			context.arc(this.pos.x, this.pos.y, this.radius, 0, this.arc360);
			context.lineWidth = this.strokeSize;
			context.strokeStyle = this.color;
			context.stroke();
		},
		drawThumb: function() {
			var canvas = ig.system.canvas;
			var context = ig.system.context;
			context.beginPath();
			context.arc(this.thumbPos.x, this.thumbPos.y, this.thumbRadius, 0, this.arc360);
			context.lineWidth = this.strokeSize;
			context.strokeStyle = this.thumbColor;
			context.stroke();
		}
	});
	JoystickArea = AButton.extend({
		joystick: null,
		pressed: function() {
			ig.input.presses[this.action] = true;
		},
		mouseDown: function() {
			this.isPressed = true;
		},
		clearMouseDown: function(id) {
			var touchController = new TouchControls(ig.system.canvas);
			if (touchController.joystick && id == touchController.joystick.touchId) {
				delete(touchController.joystick);
				ig.input.actions['left'] = false;
				ig.input.actions['right'] = false;
			}
			this.isPressed = false;
		},
		checkIfClicked: function(x, y) {
			if (x > this.pos.x && x < this.pos.x + this.size.x && y > this.pos.y && y < this.pos.y + this.size.y) {
				this.pressed();
			}
		},
		checkIfMouseDown: function(x, y, id) {
			if (x > this.pos.x && x < this.pos.x + this.size.x && y > this.pos.y && y < this.pos.y + this.size.y) {
				var touchController = new TouchControls(ig.system.canvas);
				touchController.makeJoystick(x, y, id);
				this.isPressed = true;
			}
		},
		checkIfMouseUp: function(x, y, id) {
			this.clearMouseDown(id);
		}
	});
	ig.Game.inject({
		spawnTouchEntity: function(type, x, y, settings) {
			var touchController = new TouchControls(ig.system.canvas);
			var entityClass = typeof(type) === 'string' ? ig.global[type] : type;
			if (!entityClass) {
				throw ("can't spawn touch entity of type " + type);
			}
			var ent = ig.game.spawnEntity(entityClass, x, y, settings);
			if (ent.touchable) {
				touchController.touchEntities.push(ent);
				ig.log("entity is touchable");
			}
			return ent;
		},
	})
	TouchControls.instance = null;
});